@@ -4,7 +4,7 @@ from office365.runtime.http.request_options import RequestOptions
 from office365.runtime.queries.service_operation_query import ServiceOperationQuery
 from office365.runtime.resource_path import ResourcePath
 from office365.runtime.resource_path_service_operation import ResourcePathServiceOperation
-from office365.sharepoint.actions.download_file import DownloadFileQuery
+from office365.sharepoint.actions.download_file import create_download_file_query
 from office365.sharepoint.base_entity import BaseEntity
 from office365.sharepoint.directory.user import User
 from office365.sharepoint.files.file_version_collection import FileVersionCollection
@@ -271,8 +271,7 @@ class File(AbstractFile):
         :type server_relative_url: str
         :type content: str
         """
-        url = r"{0}web/getFileByServerRelativePath(DecodedUrl='{1}')/\$value".format(
-            ctx.service_root_url(), server_relative_url)
+        url = r"web/getFileByServerRelativePath(DecodedUrl='{0}')/\$value".format(server_relative_url)
         request = RequestOptions(url)
         request.method = HttpMethod.Post
         request.set_header('X-HTTP-Method', 'PUT')
@@ -289,8 +288,7 @@ class File(AbstractFile):
         :type server_relative_url: str
         :return Response
         """
-        url = r"{0}web/getFileByServerRelativePath(DecodedUrl='{1}')/\$value".format(ctx.service_root_url(),
-                                                                                     server_relative_url)
+        url = r"web/getFileByServerRelativePath(DecodedUrl='{0}')/\$value".format(server_relative_url)
         request = RequestOptions(url)
         request.method = HttpMethod.Get
         response = ctx.execute_request_direct(request)
@@ -302,10 +300,10 @@ class File(AbstractFile):
         """
 
         def _download_inner():
-            qry = DownloadFileQuery(self.context.web, self.serverRelativeUrl, file_object)
+            qry = create_download_file_query(self, file_object)
             self.context.add_query(qry)
 
-        self.ensure_property("ServerRelativeUrl", _download_inner)
+        self.ensure_property("ServerRelativePath", _download_inner)
         return self
 
     def download_session(self, file_object, chunk_downloaded=None, chunk_size=1024 * 1024):
@@ -315,21 +313,33 @@ class File(AbstractFile):
         :type chunk_size: int
         """
 
-        def _download_inner():
-            request = RequestOptions(
-                r"{0}web/getFileByServerRelativeUrl('{1}')/\$value".format(self.context.service_root_url(),
-                                                                           self.serverRelativeUrl))
-            request.stream = True
-            response = self.context.execute_request_direct(request)
-            response.raise_for_status()
-            bytes_read = 0
-            for chunk in response.iter_content(chunk_size=chunk_size):
-                bytes_read += len(chunk)
-                if callable(chunk_downloaded):
-                    chunk_downloaded(bytes_read)
-                file_object.write(chunk)
+        def _download_as_stream():
+            qry = ServiceOperationQuery(self, "$value")
 
-        self.ensure_property("ServerRelativeUrl", _download_inner)
+            def _construct_download_request(request):
+                """
+                :type request: office365.runtime.http.request_options.RequestOptions
+                """
+                request.stream = True
+                request.method = HttpMethod.Get
+            self.context.before_execute(_construct_download_request)
+
+            def _process_download_response(response):
+                """
+                :type response: requests.Response
+                """
+                response.raise_for_status()
+                bytes_read = 0
+                for chunk in response.iter_content(chunk_size=chunk_size):
+                    bytes_read += len(chunk)
+                    if callable(chunk_downloaded):
+                        chunk_downloaded(bytes_read)
+                    file_object.write(chunk)
+            self.context.after_execute(_process_download_response)
+
+            self.context.add_query(qry)
+
+        self.ensure_property("ServerRelativeUrl", _download_as_stream)
         return self
 
     @property
