@@ -1,4 +1,4 @@
-from office365.runtime.queries.delete_entity_query import DeleteEntityQuery
+from office365.runtime.queries.service_operation_query import ServiceOperationQuery
 from office365.runtime.resource_path import ResourcePath
 from office365.sharepoint.alerts.alert_collection import AlertCollection
 from office365.sharepoint.principal.principal import Principal
@@ -9,16 +9,17 @@ class User(Principal):
     """Represents a user in Microsoft SharePoint Foundation. A user is a type of SP.Principal."""
 
     def get(self):
+        """
+        :rtype: User
+        """
         return super(User, self).get()
 
     @property
     def groups(self):
         """Gets a collection of group objects that represents all of the groups for the user."""
-        if self.is_property_available('Groups'):
-            return self.properties['Groups']
-        else:
-            from office365.sharepoint.principal.group_collection import GroupCollection
-            return GroupCollection(self.context, ResourcePath("Groups", self.resource_path))
+        from office365.sharepoint.principal.group_collection import GroupCollection
+        return self.properties.get('Groups',
+                                   GroupCollection(self.context, ResourcePath("Groups", self.resource_path)))
 
     @property
     def alerts(self):
@@ -37,9 +38,7 @@ class User(Principal):
          user's name identifier."""
         return self.properties.get('UserId', UserIdInfo())
 
-    def delete_object(self):
-        """Deletes the user."""
-        qry = DeleteEntityQuery(self)
+    def expire(self):
+        qry = ServiceOperationQuery(self, "Expire")
         self.context.add_query(qry)
-        self.remove_from_parent_collection()
         return self
