@@ -1,11 +1,9 @@
 # coding=utf-8
 from office365.runtime.client_result import ClientResult
 from office365.runtime.client_value_collection import ClientValueCollection
-from office365.runtime.queries.client_query import ClientQuery
 from office365.runtime.queries.service_operation_query import ServiceOperationQuery
 from office365.runtime.resource_path import ResourcePath
 from office365.runtime.resource_path_service_operation import ResourcePathServiceOperation
-from office365.sharepoint.actions.getWebUrlFromPage import create_get_web_url_from_page_url_query
 from office365.sharepoint.alerts.alert_collection import AlertCollection
 from office365.sharepoint.changes.change_collection import ChangeCollection
 from office365.sharepoint.clientsidecomponent.types import SPClientSideComponentQueryResult
@@ -13,6 +11,7 @@ from office365.sharepoint.contenttypes.content_type_collection import ContentTyp
 from office365.sharepoint.eventreceivers.event_receiver_definition import EventReceiverDefinitionCollection
 from office365.sharepoint.fields.field_collection import FieldCollection
 from office365.sharepoint.files.file import File
+from office365.sharepoint.flows.flow_synchronization_result import FlowSynchronizationResult
 from office365.sharepoint.folders.folder import Folder
 from office365.sharepoint.folders.folder_collection import FolderCollection
 from office365.sharepoint.listitems.listitem import ListItem
@@ -82,9 +81,46 @@ class Web(SecurableObject):
         :type context: office365.sharepoint.client_context.ClientContext
         :type page_full_url: str
         """
-        qry = create_get_web_url_from_page_url_query(context, page_full_url)
+        result = ClientResult(context)
+        payload = {
+            "pageFullUrl": page_full_url
+        }
+        qry = ServiceOperationQuery(context.web, "GetWebUrlFromPageUrl", None, payload, None, result)
+        qry.static = True
         context.add_query(qry)
-        return qry.return_type
+        return result
+
+    def create_group_based_environment(self):
+        return_type = FlowSynchronizationResult(self.context)
+        qry = ServiceOperationQuery(self, "CreateGroupBasedEnvironment", None, None, None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
+    def get_group_based_environment(self):
+        return_type = FlowSynchronizationResult(self.context)
+        qry = ServiceOperationQuery(self, "GetGroupBasedEnvironment", None, None, None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
+    def sync_flow_instances(self, target_web_url):
+        """
+        :param str target_web_url:
+        """
+        return_type = FlowSynchronizationResult(self.context)
+        payload = {"targetWebUrl": target_web_url}
+        qry = ServiceOperationQuery(self, "SyncFlowInstances", None, payload, None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
+    def sync_flow_templates(self, category):
+        """
+        :param str category:
+        """
+        return_type = FlowSynchronizationResult(self.context)
+        payload = {"category": category}
+        qry = ServiceOperationQuery(self, "SyncFlowTemplates", None, payload, None, return_type)
+        self.context.add_query(qry)
+        return return_type
 
     def get_all_client_side_components(self):
         result = ClientResult(self.context)
@@ -92,10 +128,10 @@ class Web(SecurableObject):
         self.context.add_query(qry)
         return result
 
-    def get_client_side_web_parts(self, project, includeErrors=False):
+    def get_client_side_web_parts(self, project, include_errors=False):
         result = ClientValueCollection(SPClientSideComponentQueryResult)
         params = {
-            "includeErrors": includeErrors,
+            "includeErrors": include_errors,
             "project": project
         }
         qry = ServiceOperationQuery(self, "getClientSideWebParts", None, params, None, result)
@@ -140,29 +176,34 @@ class Web(SecurableObject):
         return result
 
     def get_all_webs(self):
-        """Returns a collection containing a flat list of all Web objects in the Web object."""
-        result = ClientResult(self.context, self.webs)
-        qry = ClientQuery(self.context, self.webs, None, None, result)
-        self.context.add_query(qry)
+        """Returns a collection containing a flat list of all Web objects in the Web."""
+        from office365.sharepoint.webs.web_collection import WebCollection
+        result = ClientResult(self.context, WebCollection(self.context, self.webs.resource_path))
 
-        def _load_sub_webs(resp):
-            self._load_sub_webs_inner(result.value)
+        def _webs_loaded():
+            self._load_sub_webs_inner(self.webs, result)
 
-        self.context.after_execute(_load_sub_webs)
+        self.ensure_property("Webs", _webs_loaded)
         return result
 
-    def _load_sub_webs_inner(self, webs, result=None):
-        if result is None:
-            result = webs
-        for parent_web in webs:
-            sub_webs = parent_web.webs
-            self.context.load(sub_webs)
-            self.context.execute_query()
-            for web in sub_webs:
-                result.add_child(web)
-            self._load_sub_webs_inner(sub_webs, result)
+    def _load_sub_webs_inner(self, webs, result):
+        """
+        :type webs: office365.sharepoint.webs.web_collection.WebCollection
+        :type result: ClientResult
+        """
+        for cur_web in webs:  # type: Web
+            result.value.add_child(cur_web)
+
+            def _webs_loaded(web):
+                if len(web.webs) > 0:
+                    self._load_sub_webs_inner(web.webs, result)
+
+            cur_web.ensure_property("Webs", _webs_loaded, cur_web)
 
     def get_list_using_path(self, decoded_url):
+        """
+        :type decoded_url: str
+        """
         return_list = List(self.context)
         self.lists.add_child(return_list)
         from office365.sharepoint.types.resource_path import ResourcePath as SPResPath
@@ -176,9 +217,12 @@ class Web(SecurableObject):
         self.context.add_query(qry)
         return return_type
 
-    def get_sharing_link_data(self, linkUrl):
+    def get_sharing_link_data(self, link_url):
+        """
+        :type link_url: str
+        """
         result = SharingLinkData()
-        qry = ServiceOperationQuery(self, "GetSharingLinkData", [linkUrl], None, None, result)
+        qry = ServiceOperationQuery(self, "GetSharingLinkData", [link_url], None, None, result)
         self.context.add_query(qry)
         return result
 
@@ -279,7 +323,7 @@ class Web(SecurableObject):
         :type decoded_url: str
         """
         params = {"DecodedUrl": decoded_url}
-        return File(
+        return Folder(
             self.context,
             ResourcePathServiceOperation("getFolderByServerRelativePath", params, self.resource_path)
         )
@@ -467,6 +511,7 @@ class Web(SecurableObject):
 
         def _web_initialized():
             result.value = Web.unshare_object(self.context, self.url)
+
         self.ensure_property("Url", _web_initialized)
         return result.value
 
@@ -517,9 +562,9 @@ class Web(SecurableObject):
         :param (str) -> None on_resolved:
         """
         options = {
-            ExternalSharingSiteOption.View: context.web.associatedVisitorGroup,
-            ExternalSharingSiteOption.Edit: context.web.associatedMemberGroup,
-            ExternalSharingSiteOption.Owner: context.web.associatedOwnerGroup,
+            ExternalSharingSiteOption.View: context.web.associated_visitor_group,
+            ExternalSharingSiteOption.Edit: context.web.associated_member_group,
+            ExternalSharingSiteOption.Owner: context.web.associated_owner_group,
         }
         grp = options[share_option]
         context.load(grp)
@@ -636,21 +681,15 @@ class Web(SecurableObject):
     @property
     def webs(self):
         """Get child webs"""
-        if self.is_property_available('Webs'):
-            return self.properties['Webs']
-        else:
-            from office365.sharepoint.webs.web_collection import WebCollection
-            parent_web_url = None
-            if self.is_property_available('Url'):
-                parent_web_url = self.properties['Url']
-            return WebCollection(self.context,
-                                 ResourcePath("webs", self.resource_path), parent_web_url)
+        from office365.sharepoint.webs.web_collection import WebCollection
+        return self.properties.get("Webs",
+                                   WebCollection(self.context, ResourcePath("webs", self.resource_path), self))
 
     @property
     def folders(self):
         """Get folder resources"""
         return self.properties.get('Folders',
-                                   FolderCollection(self.context, ResourcePath("folders", self.resource_path)))
+                                   FolderCollection(self.context, ResourcePath("folders", self.resource_path), self))
 
     @property
     def lists(self):
@@ -683,19 +722,19 @@ class Web(SecurableObject):
                                    Web(self.context, ResourcePath("ParentWeb", self.resource_path)))
 
     @property
-    def associatedVisitorGroup(self):
+    def associated_visitor_group(self):
         """Gets or sets the associated visitor group of the Web site."""
         return self.properties.get('AssociatedVisitorGroup',
                                    Group(self.context, ResourcePath("AssociatedVisitorGroup", self.resource_path)))
 
     @property
-    def associatedOwnerGroup(self):
+    def associated_owner_group(self):
         """Gets or sets the associated owner group of the Web site."""
         return self.properties.get('AssociatedOwnerGroup',
                                    Group(self.context, ResourcePath("AssociatedOwnerGroup", self.resource_path)))
 
     @property
-    def associatedMemberGroup(self):
+    def associated_member_group(self):
         """Gets or sets the group of users who have been given contribute permissions to the Web site."""
         return self.properties.get('AssociatedMemberGroup',
                                    Group(self.context, ResourcePath("AssociatedMemberGroup", self.resource_path)))
@@ -858,27 +897,25 @@ class Web(SecurableObject):
         """
         return self.properties.get("ServerRelativePath", SPResPath(None))
 
-    def get_property(self, name):
-        if name == "ContentTypes":
-            return self.content_types
-        elif name == "RootFolder":
-            return self.root_folder
-        elif name == "RegionalSettings":
-            return self.regional_settings
-        elif name == "RoleDefinitions":
-            return self.role_definitions
-        elif name == "RecycleBin":
-            return self.recycle_bin
-        elif name == "CurrentUser":
-            return self.current_user
-        elif name == "AvailableFields":
-            return self.available_fields
-        else:
-            return super(Web, self).get_property(name)
+    def get_property(self, name, default_value=None):
+        if default_value is None:
+            property_mapping = {
+                "ContentTypes": self.content_types,
+                "RootFolder": self.root_folder,
+                "RegionalSettings": self.regional_settings,
+                "RoleDefinitions": self.role_definitions,
+                "RecycleBin": self.recycle_bin,
+                "CurrentUser": self.current_user,
+                "AvailableFields": self.available_fields,
+                "AssociatedOwnerGroup": self.associated_owner_group,
+                "AssociatedMemberGroup": self.associated_member_group,
+                "AssociatedVisitorGroup": self.associated_visitor_group
+            }
+            default_value = property_mapping.get(name, None)
+        return super(Web, self).get_property(name, default_value)
 
     def set_property(self, name, value, persist_changes=True):
         super(Web, self).set_property(name, value, persist_changes)
-        # fallback: create a new resource path
         if name == "Url":
             self._web_url = value
         return self
